# $Id: dinfo.rb,v 1.1 2003/07/29 14:33:47 sdalu Exp $

# 
# CONTACT     : zonecheck@nic.fr
# AUTHOR      : Stephane D'Alu <sdalu@nic.fr>
#
# CREATED     : 2003/07/22 13:45:58
# REVISION    : $Revision: 1.1 $ 
# DATE        : $Date: 2003/07/29 14:33:47 $
#
# CONTRIBUTORS: (see also CREDITS file)
#
#
# LICENSE     : GPL v2 (or MIT/X11-like after agreement)
# COPYRIGHT   : AFNIC (c) 2003
#
# This file is part of ZoneCheck.
#
# ZoneCheck is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# ZoneCheck is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ZoneCheck; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#


class DInfo
    class Domain
        def initialize(name=nil, ns=nil)
            self.name = name unless name.nil?
            self.ns   = ns   unless ns.nil?
        end

        attr_reader :name, :ns_list

        def name=(domain)
            @name = domain
        end

	def ns
	    return nil if @ns_list.nil?
	    @ns_list.collect { |host, ips|
		ips.empty? ? host : "#{host}=#{ips.join(',')}" }.join(';')
	end
        
        def ns=(ns)
            return @ns_list = nil if ns.nil?

            # Parse inputed NS (and IPs)
            @ns_list = [ ]
            ns.split(/\s*;\s*/).each { |entry|
                ips  = []
                if entry =~ /^(.*?)\s*=\s*(.*)$/
                    host, ips_str = $1, $2
                    ips_str.split(/\s*,\s*|\s+/).each { |str|
                        ips << str }
                else
                    host = entry
                end
                @ns_list << [ host, ips ]
            }

            # 
            @ns_list
        end
    end

    class Status
	attr_writer :command, :exitcode, :signal
	attr_reader :command, :exitcode, :signal

	attr_writer :error, :warning, :passed
	attr_reader :error, :warning, :passed


	attr_writer :garbage, :skipped, :saved
	attr_reader :garbage, :skipped, :saved
    end


    class Option
        def initialize
            @opt = { }
        end
        
	def [](key)		; @opt[key]			; end
	def []=(key,value)	; @opt[key] = value		; end
	def delete(key)		; @opt.delete(key)		; end
	def clear		; @opt = { }			; end
	def each		; @opt.each { |*a| yield a }	; end 

        def <<(args)
            args.strip.split(/\s*,\s*/).each { |arg|
                case arg
                when /^-$/              then @opt = { }
                when /^-(\w+)$/         then @opt.delete($1)
                when /^\+?(\w+)$/       then @opt[$1] = true
                when /^\+?(\w+)=(\w+)$/ then @opt[$1] = $2
                else raise ArgumentError, "Bad option specification"
                end
            }
            self
        end
    end

    attr_reader :domain, :option, :status
    attr_writer :status

    def initialize
	@domain = Domain::new
	@option = Option::new
	@status = nil
    end
end
