# $Id: manager.rb,v 1.1 2003/07/29 14:33:47 sdalu Exp $

# 
# CONTACT     : zonecheck@nic.fr
# AUTHOR      : Stephane D'Alu <sdalu@nic.fr>
#
# CREATED     : 2003/07/22 13:45:58
# REVISION    : $Revision: 1.1 $ 
# DATE        : $Date: 2003/07/29 14:33:47 $
#
# CONTRIBUTORS: (see also CREDITS file)
#
#
# LICENSE     : GPL v2 (or MIT/X11-like after agreement)
# COPYRIGHT   : AFNIC (c) 2003
#
# This file is part of ZoneCheck.
#
# ZoneCheck is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# ZoneCheck is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ZoneCheck; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#


require 'thread'
require 'launch'

class Manager
    attr_reader :queue

    attr_reader :ctrl, :input, :backend

    def initialize
	@queue = Queue::new
	@thr_input  = nil
	@thr_worker = [ ]
    end

    def ctrl=(ctrl)
	@ctrl = ctrl
    end

    def input=(input)
	@input = input
    end

    def backend=(backend)
	@backend = backend
    end

    def start
	@thr_input = Thread::new {
	    begin
		@input.init
		while @input.more_data?
		    @queue << input.readconf
		end
		@input.destroy
	    rescue => e
		puts "EXCEPTION: #{e}"
		puts e.backtrace.join("\n")
	    end
	}

	THREAD_COUNT.times { |thridx|
	    @thr_worker << Thread::new {
		    
		while true
		    # Jitter
		    sleep(rand(2))
		    
		    # Retrieve information about the zone to test
		    dinfo = @queue.pop(false)
		    
		    status = @backend.process(dinfo)
		    
		    # Log information summary
		    @ctrl.log(dinfo)
		end
	    }
	}

    end

    def join
	@thr_input.join
	puts "TOTAL SIZE=#{@queue.size}"
	@thr_worker.each { |thr| thr.join }
	io.close
    end

end
