# $Id: param.rb,v 1.2 2003/08/07 13:12:52 sdalu Exp $

# 
# CONTACT     : zonecheck@nic.fr
# AUTHOR      : Stephane D'Alu <sdalu@nic.fr>
#
# CREATED     : 2003/07/22 13:45:58
# REVISION    : $Revision: 1.2 $ 
# DATE        : $Date: 2003/08/07 13:12:52 $
#
# CONTRIBUTORS: (see also CREDITS file)
#
#
# LICENSE     : GPL v2 (or MIT/X11-like after agreement)
# COPYRIGHT   : AFNIC (c) 2003
#
# This file is part of ZoneCheck.
#
# ZoneCheck is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# ZoneCheck is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ZoneCheck; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

class Param
    ##
    ## Hold information about file system
    ##
    ## cfgfile: configuration file to use (zc.conf)
    ## testdir: directory where tests are located
    ##
    class FSData
	attr_reader :logfile
	attr_writer :logfile
	
	def initialize
	    @cfgfile	= ZC_CONFIG_FILE
	    @testdir	= ZC_TEST_DIR
	end

	def autoconf
	    # Debug
	    $dbg.msg(DBG::AUTOCONF, "configuration file: #{@cfgfile}")
	    $dbg.msg(DBG::AUTOCONF, "tests directory: #{@testdir}")
	end
    end

    ##
    ## Hold optionnal information
    ##
    class Option
	def initialize
	    @opt	= { }
	end
	
	def [](key)		; @opt[key]			; end
	def []=(key,value)	; @opt[key] = value		; end
	def delete(key)		; @opt.delete(key)		; end
	def clear		; @opt = { }			; end
	def each		; @opt.each { |*a| yield a }	; end 

	def <<(args)
	    args = args.strip
	    while !args.empty? do
		unless args =~ /^\s*([+!\-])?(?:(\w+)(?:=((?:'[^']*'|"(?:[^"\\]|\\.)*"|[^'",]+)*))?)\s*(?:,|$)/
		    raise ArgumentError, 'bad option specification'
		end
		op, key, value, args = $1, $2, $3, $'


		case op
		when '-', '!'
		    if !value.nil?
			raise ArgumentError, 
			    'negative option doesn\'t take a value'
		    end
		    if key.nil?
		    then self.clear
		    else self.delete(key)
		    end
		when '+', nil
		    if value.nil?
			if key.nil?
			then raise ArgumentError, 'option \'+\' doesn\'t exist'
			else self[key] = true
			end
		    else
			string = ""
			while !value.empty?
			    value =~ /^(?:'[^']*'|"(?:[^"\\]|\\.)*"|[^'"]+)/
			    value = $'
			    case $&[0]
			    when ?"
				tmp = $&
				tmp.gsub!(/\\./, '\1')
				string += tmp[1..-2]
			    when ?'
				string += $&[1..-2]
			    else 
				string += $&
			    end
			end
			self[key] = string
		    end
		end
	    end

	    self
	end
    end



    attr_reader :input, :backend, :controler
    def initialize
	@input		= Option::new
	@backend	= Option::new
	@controler	= Option::new
    end

end
